@@ -1,4 +1,4 @@
-// Copyright (C) 2016 - 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (C) 2016 - 2023 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 #include <stdexcept>
 #include <vector>
 
-#include "../rocFFT/clients/fft_params.h"
+#include "../rocFFT/shared/fft_params.h"
 
 #include "accuracy_test.h"
 #include "fftw_transform.h"
@@ -39,6 +39,9 @@ const static std::vector<size_t> pow2_range
        8388608, 16777216, 33554432, 67108864, 134217728, 268435456, 536870912};
 // 2^30 is 1073741824;
 
+const static std::vector<size_t> pow2_range_half
+    = {2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536};
+
 const static std::vector<size_t> pow3_range = {3,
                                                9,
                                                27,
@@ -113,7 +116,7 @@ const static std::vector<std::vector<size_t>> ooffset_range = {{0, 0}, {1, 1}};
 INSTANTIATE_TEST_SUITE_P(pow2_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({pow2_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -126,7 +129,34 @@ INSTANTIATE_TEST_SUITE_P(pow2_1D,
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow2_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({pow2_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
+                                                             batch_range_1D,
+                                                             stride_range,
+                                                             stride_range,
+                                                             ioffset_range,
+                                                             ooffset_range,
+                                                             place_range,
+                                                             false,
+                                                             false)),
+                         accuracy_test::TestName);
+
+INSTANTIATE_TEST_SUITE_P(pow2_1D_half,
+                         accuracy_test,
+                         ::testing::ValuesIn(param_generator(generate_lengths({pow2_range_half}),
+                                                             {fft_precision_half},
+                                                             batch_range_1D,
+                                                             stride_range,
+                                                             stride_range,
+                                                             ioffset_range_zero,
+                                                             ooffset_range_zero,
+                                                             place_range,
+                                                             false,
+                                                             false)),
+                         accuracy_test::TestName);
+INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow2_1D_half,
+                         accuracy_test,
+                         ::testing::ValuesIn(param_generator(generate_lengths({pow2_range_half}),
+                                                             {fft_precision_half},
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -140,7 +170,7 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow2_1D,
 INSTANTIATE_TEST_SUITE_P(pow3_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({pow3_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -153,7 +183,7 @@ INSTANTIATE_TEST_SUITE_P(pow3_1D,
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow3_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({pow3_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -167,7 +197,7 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow3_1D,
 INSTANTIATE_TEST_SUITE_P(pow5_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({pow5_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -180,7 +210,7 @@ INSTANTIATE_TEST_SUITE_P(pow5_1D,
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow5_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({pow5_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -194,7 +224,7 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow5_1D,
 INSTANTIATE_TEST_SUITE_P(radX_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({radX_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -207,7 +237,7 @@ INSTANTIATE_TEST_SUITE_P(radX_1D,
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_radX_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({radX_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -221,7 +251,7 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_radX_1D,
 INSTANTIATE_TEST_SUITE_P(prime_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({prime_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -234,7 +264,7 @@ INSTANTIATE_TEST_SUITE_P(prime_1D,
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_prime_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({prime_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -248,7 +278,7 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_prime_1D,
 INSTANTIATE_TEST_SUITE_P(mix_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({mix_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -261,7 +291,7 @@ INSTANTIATE_TEST_SUITE_P(mix_1D,
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_mix_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator(generate_lengths({mix_range}),
-                                                             precision_range,
+                                                             precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
                                                              stride_range,
@@ -307,14 +337,16 @@ INSTANTIATE_TEST_SUITE_P(small_1D,
 //
 // The below test covers non-unit strides, pow of 2, middle sizes, which has SBCC/SBRC kernels
 // invloved.
-const static std::vector<size_t>              pow2_range_for_stride  = {4096, 8192, 524288};
-const static std::vector<std::vector<size_t>> stride_range_for_pow2  = {{2}, {3}};
-const static std::vector<size_t>              batch_range_for_stride = {2, 1};
+const static std::vector<size_t>              pow2_range_for_stride      = {4096, 8192, 524288};
+const static std::vector<size_t>              pow2_range_for_stride_half = {4096, 8192};
+const static std::vector<std::vector<size_t>> stride_range_for_pow2      = {{2}, {3}};
+const static std::vector<size_t>              batch_range_for_stride     = {2, 1};
+
 INSTANTIATE_TEST_SUITE_P(
     pow2_1D_stride_complex,
     accuracy_test,
     ::testing::ValuesIn(param_generator_complex(generate_lengths({pow2_range_for_stride}),
-                                                precision_range,
+                                                precision_range_sp_dp,
                                                 batch_range_1D,
                                                 stride_range_for_pow2,
                                                 stride_range_for_pow2,
@@ -329,7 +361,22 @@ INSTANTIATE_TEST_SUITE_P(
     pow2_1D_stride_real,
     accuracy_test,
     ::testing::ValuesIn(param_generator_real(generate_lengths({pow2_range_for_stride}),
-                                             precision_range,
+                                             precision_range_sp_dp,
+                                             batch_range_1D,
+                                             stride_range_for_pow2,
+                                             stride_range_for_pow2,
+                                             ioffset_range_zero,
+                                             ooffset_range_zero,
+                                             place_range,
+                                             false,
+                                             false)),
+    accuracy_test::TestName);
+
+INSTANTIATE_TEST_SUITE_P(
+    pow2_1D_stride_real_half,
+    accuracy_test,
+    ::testing::ValuesIn(param_generator_real(generate_lengths({pow2_range_for_stride_half}),
+                                             {fft_precision_half},
                                              batch_range_1D,
                                              stride_range_for_pow2,
                                              stride_range_for_pow2,
@@ -351,8 +398,11 @@ inline auto
 
     std::vector<fft_params> params;
 
-    for(auto& transform_type :
-        {fft_transform_type_complex_forward, fft_transform_type_complex_inverse})
+    // for(auto& transform_type :
+    // {fft_transform_type_complex_forward, fft_transform_type_complex_inverse})
+    // {
+
+    for(auto& transform_type : trans_type_range_complex)
     {
         for(const auto& lengths : v_lengths)
         {
@@ -404,7 +454,7 @@ INSTANTIATE_TEST_SUITE_P(
     pow2_1D_complex_batched_2D_strided,
     accuracy_test,
     ::testing::ValuesIn(param_generator_complex_1d_batched_2d(generate_lengths({pow2_range_2D}),
-                                                              precision_range,
+                                                              precision_range_sp_dp,
                                                               ioffset_range_zero,
                                                               ooffset_range_zero,
                                                               place_range)),
@@ -415,7 +465,7 @@ INSTANTIATE_TEST_SUITE_P(
     pow3_1D_complex_batched_2D_strided,
     accuracy_test,
     ::testing::ValuesIn(param_generator_complex_1d_batched_2d(generate_lengths({pow3_range_2D}),
-                                                              precision_range,
+                                                              precision_range_sp_dp,
                                                               ioffset_range_zero,
                                                               ooffset_range_zero,
                                                               place_range)),
@@ -426,7 +476,7 @@ INSTANTIATE_TEST_SUITE_P(
     pow5_1D_complex_batched_2D_strided,
     accuracy_test,
     ::testing::ValuesIn(param_generator_complex_1d_batched_2d(generate_lengths({pow5_range_2D}),
-                                                              precision_range,
+                                                              precision_range_sp_dp,
                                                               ioffset_range_zero,
                                                               ooffset_range_zero,
                                                               place_range)),
@@ -438,7 +488,7 @@ INSTANTIATE_TEST_SUITE_P(
     prime_1D_complex_batched_2D_strided,
     accuracy_test,
     ::testing::ValuesIn(param_generator_complex_1d_batched_2d(generate_lengths({prime_range_2D}),
-                                                              precision_range,
+                                                              precision_range_sp_dp,
                                                               ioffset_range_zero,
                                                               ooffset_range_zero,
                                                               place_range)),
